@@ -7,14 +7,14 @@ use IO::Socket::SIPC;
 my $sipc = IO::Socket::SIPC->new( favorite => 'IO::Socket::INET' );
 
 $sipc->connect(
-   PeerAddr        => 'localhost',
-   PeerPort        => 50010,
-   Proto           => 'tcp',
+   PeerAddr => 'localhost',
+   PeerPort => 50010,
+   Proto    => 'tcp',
 ) or die $sipc->errstr($@);
 
 warn "client connected to server\n";
 
-$sipc->send("Hello server, gimme some data :-)\n") or die $sipc->errstr($!);
+$sipc->send("Hello server, gimme some data :-)\n", 1) or die $sipc->errstr($!);
 my $answer = $sipc->read or die $sipc->errstr($!);
 warn "server data: \n";
 warn Dumper($answer);
