@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use IO::Socket::SIPC;
 
-my $sipc = IO::Socket::SIPC->new();
+my $sipc = IO::Socket::SIPC->new( favorite => 'IO::Socket::SSL' );
 
 $sipc->connect(
    LocalAddr       => 'localhost',
@@ -25,10 +25,10 @@ $sipc->sock->timeout(10);
 while ( 1 ) {
    while (my $client = $sipc->accept()) {
       print "connect from client: ", $client->sock->peerhost, "\n";
-      my $request = $client->read or die $client->errstr($client->sock->errstr);
-      next unless $$request;
-      chomp($$request);
-      warn "client says: $$request\n";
+      my $request = $client->read(1) or die $client->errstr($client->sock->errstr);
+      next unless $request;
+      chomp($request);
+      warn "client says: $request\n";
       $client->send({ foo => 'is foo', bar => 'is bar', baz => 'is baz'}) or die $client->errstr($client->sock->errstr);
       $client->disconnect or die $client->errstr($!);
    }
